package cmdbuild.elements;

import cmdbuild.db.DBService;
import cmdbuild.collection.DomainBean;
import cmdbuild.collection.HistoryBean;
import cmdbuild.collection.RelationBean;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.Vector;

public class CMDBmap {

	String CREATE_DOMAIN = "{ ?= call createDomain(?,?,?,?,?,?,?) }"; // Name,class1,class2,directdescrition,inversedescription,cardinality,status
	String MODIFY_DOMAIN = "{ ?= call modifyDomain(?,?,?,?,?,?,?,?) }"; // Name,newName,class1,class2,directdescrition,inversedescription,cardinality,status
	String DELETE_DOMAIN = "{ ?= call deleteDomain(?) }"; // Name
	
	private DBService db;
	
	public CMDBmap() {
		super();
		// TODO Auto-generated constructor stub
	}
	
	public CMDBmap(DBService db) {
		super();
		// TODO Auto-generated constructor stub
		this.db = db;
	}

	public boolean isEmpy(String name) throws SQLException{
		// adding Map_ beacause isempty is used by classes and domains too
		if (name.indexOf("Map")==-1){
			name = "Map_"+name;
		}
		boolean empty = false;
		String query = "SELECT isempty('"+name+"');";
		ResultSet rs = null;
		PreparedStatement stm = null;
		System.out.println("Query ISEMPTY="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				if ( rs.next() ) { // while ( rs.next() ) {
					empty = rs.getBoolean(1);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally { // added**
			DBService.close(rs, stm, con);
		}
		return empty;
	}

	// **********
	// **domain**
	// **********
	public boolean createDomain(DomainBean bean) throws SQLException {
		boolean ok = false;
		CallableStatement stm = null;
		System.out.println("createdomain("+bean.getName()+","+bean.getStartClass()+","+bean.getEndClass()+","+bean.getDirectDescription()+","+bean.getInverseDescription()+","+bean.getStatus()+")");
		Connection con = db.getConnection();
		try {
			stm = con.prepareCall(CREATE_DOMAIN);
			stm.registerOutParameter(1,Types.BOOLEAN);
			stm.setString(2,bean.getName());
			stm.setString(3,bean.getStartClass());
			stm.setString(4,bean.getEndClass());
			stm.setString(5,bean.getDirectDescription());
			stm.setString(6,bean.getInverseDescription());
			stm.setString(7,bean.getCardinality());
			stm.setString(8,bean.getStatus());
			stm.execute();
			ok = true;
		} catch (SQLException ex) {
			throw ex;
		} finally {
			DBService.close(null, stm, con);
		}
		return ok;
	}
	public boolean modifyDomain(String newname, DomainBean bean) throws SQLException {
		boolean ok = false;
		CallableStatement stm = null;
		System.out.println("modifydomain("+bean.getName()+","+newname+","+bean.getStartClass()+","+bean.getEndClass()+","+bean.getDirectDescription()+","+bean.getInverseDescription()+","+bean.getStatus()+")");
		Connection con = db.getConnection();
		try {
			stm = con.prepareCall(MODIFY_DOMAIN);
			stm.registerOutParameter(1,Types.BOOLEAN);
			stm.setString(2,bean.getName());
			stm.setString(3,newname);
			stm.setString(4,bean.getStartClass());
			stm.setString(5,bean.getEndClass());
			stm.setString(6,bean.getDirectDescription());
			stm.setString(7,bean.getInverseDescription());
			stm.setString(8,bean.getCardinality());
			stm.setString(9,bean.getStatus());
			stm.execute();
			ok = true;
		} catch (SQLException ex) {
			throw ex;
		} finally {
			DBService.close(null, stm, con);
		}
		return ok;
	}
	public boolean deleteDomain(String name) throws SQLException{
		boolean ok = false;
		CallableStatement stm = null;
		System.out.println("deletedomain("+name+")");
		Connection con = db.getConnection();
		try {
			stm = con.prepareCall(DELETE_DOMAIN);
			stm.registerOutParameter(1,Types.BOOLEAN);
			stm.setString(2,name);
			stm.execute();
			ok = true;
		} catch (SQLException ex) {
			throw ex;
		} finally {
			DBService.close(null, stm, con);
		}
		return ok;
	}
	public  Vector selectDomainByClass(String classname) throws SQLException {
		Vector domainBeanVector = new Vector();
		ResultSet rs = null;
		PreparedStatement stm = null;
		DomainBean obj = null;
		String query = "SELECT domainid,domainname,domaindirectdescription,domaininversedescription,domainclass1,domainclass2,domaincardinality,domainstatus FROM cmdbdomaincatalog WHERE domainmode='read' AND \"cmdbdomaincatalog\".\"domainclass1\"='"+classname+"' OR \"cmdbdomaincatalog\".\"domainclass2\"='"+classname+"'";
		System.out.println("Query SELECTDOMAINBYCLASS="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while ( rs.next() ) {
					obj = setDomainObject(rs);
					domainBeanVector.add(obj);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return domainBeanVector;
	}
	public  DomainBean selectDomainByName(String name) throws SQLException {
		ResultSet rs = null;
		PreparedStatement stm = null;
		DomainBean obj = null;
		if (name.indexOf("Map_")==-1){
			name = "Map_"+name;
		}
		String query = "SELECT domainid,domainname,domaindirectdescription,domaininversedescription,domainclass1,domainclass2,domaincardinality,domainstatus FROM cmdbdomaincatalog WHERE domainmode='read' AND \"cmdbdomaincatalog\".\"domainname\"='"+name+"'";
		System.out.println("Query SELECTDOMAINBYNAME="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				if ( rs.next() ) {
					obj = setDomainObject(rs);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return obj;
	}
	public Vector selectAllDomain() throws SQLException {
		Vector vect = new Vector();
		ResultSet rs = null;
		PreparedStatement stm = null;
		DomainBean obj = null;
		String query = "SELECT domainid,domainname,domaindirectdescription,domaininversedescription,domainclass1,domainclass2,domaincardinality,domainstatus FROM cmdbdomaincatalog WHERE domainmode='read'";
		System.out.println("Query SELECTALLDOMAIN="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while( rs.next() ) {
					obj = setDomainObject(rs);
					vect.add(obj);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return vect;
	}
	// **********
	// **relation
	// **********
	public  boolean createRelation(String domainName, int domainId, int c1, int o1, int c2, int o2, String user) throws SQLException {
		int r=0;
		boolean result;
		Statement stm = null;
		ResultSet rs = null;
		if (domainName.indexOf("Map_")==-1){
			domainName = "Map_"+domainName;
		}
		String query="";
		if (user!=null){
			query = "INSERT INTO \""+domainName+"\" (\"IdDomain\",\"IdClass1\",\"IdObj1\",\"IdClass2\",\"IdObj2\",\"Status\",\"User\") VALUES ("+domainId+","+c1+","+o1+","+c2+","+o2+",'A','"+user+"')";
		} else {
			query = "INSERT INTO \""+domainName+"\" (\"IdDomain\",\"IdClass1\",\"IdObj1\",\"IdClass2\",\"IdObj2\",\"Status\") VALUES ("+domainId+","+c1+","+o1+","+c2+","+o2+",'A')";	
		}
		System.out.println("Query CREATERELATION="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.createStatement();
				r= stm.executeUpdate(query) ;
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		if (r==1) {
			result = true;
		} else {
			result = false;
		}
		return result;
	}
	public  boolean modifyRelation(String domainName, int domainId, boolean direct, int c1, int o1, int c2, int o2, int newc, int newo, String user) throws SQLException {
		int r=0;
		boolean result;
		Statement stm = null;
		ResultSet rs = null;
		if (domainName.indexOf("Map_")==-1){
			domainName = "Map_"+domainName;
		}
		System.out.println("direct is "+direct);
		String query = "";
		if (direct) {
			if (user!=null){
				query = "UPDATE \""+domainName+"\" SET \"IdClass2\"='"+newc+"' , \"IdObj2\"='"+newo+"' ,\"User\"='"+user+"' WHERE \"IdDomain\"='"+domainId+"' AND \"IdClass1\"='"+c1+"' AND \"IdObj1\"='"+o1+"' AND \"IdClass2\"='"+c2+"' AND \"IdObj2\"='"+o2+"'";
			} else {
				query = "UPDATE \""+domainName+"\" SET \"IdClass2\"='"+newc+"' , \"IdObj2\"='"+newo+"' WHERE \"IdDomain\"='"+domainId+"' AND \"IdClass1\"='"+c1+"' AND \"IdObj1\"='"+o1+"' AND \"IdClass2\"='"+c2+"' AND \"IdObj2\"='"+o2+"'";	
			}			
		} else { // !direct = inverseDescription
			if (user!=null){ 
				query = "UPDATE \""+domainName+"\" SET \"IdClass1\"='"+newc+"' , \"IdObj1\"='"+newo+"' ,\"User\"='"+user+"' WHERE \"IdDomain\"='"+domainId+"' AND \"IdClass1\"='"+c2+"' AND \"IdObj1\"='"+o2+"' AND \"IdClass2\"='"+c1+"' AND \"IdObj2\"='"+o1+"'";	
			} else {
				query = "UPDATE \""+domainName+"\" SET \"IdClass1\"='"+newc+"' , \"IdObj1\"='"+newo+"' WHERE \"IdDomain\"='"+domainId+"' AND \"IdClass1\"='"+c2+"' AND \"IdObj1\"='"+o2+"' AND \"IdClass2\"='"+c1+"' AND \"IdObj2\"='"+o1+"'";
			}
		}
		System.out.println("Query MODIFYRELATION="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.createStatement();
				r= stm.executeUpdate(query);
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		if (r>0) {
			result = true;
		} else {
			result = false;
		}
		return result;
	}
	public  boolean deleteRelation(String domainName, int domainId, boolean direct, int c1, int o1, int c2, int o2, String user) throws SQLException {
		int r=0;
		boolean result;
		Statement stm = null;
		ResultSet rs = null;
		if (domainName.indexOf("Map_")==-1){
			domainName = "Map_"+domainName;
		}
		String query = "";
		if (direct) {
			query = "DELETE FROM \""+domainName+"\" WHERE \"IdDomain\"='"+domainId+"' AND \"IdClass1\"='"+c1+"' AND \"IdObj1\"='"+o1+"' AND \"IdClass2\"='"+c2+"' AND \"IdObj2\"='"+o2+"'";
		} else { // !direct = inverseDescription
			query = "DELETE FROM \""+domainName+"\" WHERE \"IdDomain\"='"+domainId+"' AND \"IdClass1\"='"+c2+"' AND \"IdObj1\"='"+o2+"' AND \"IdClass2\"='"+c1+"' AND \"IdObj2\"='"+o1+"'";
		}
		System.out.println("Query DELETERELATION="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.createStatement();
				r= stm.executeUpdate(query);
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		if (r>0) {
			result = true;
		} else {
			result = false;
		}
		return result;
	}
	// ************
	// **relation**
	// ************
	public Vector selectRelationTips(String couple_class_obj) throws SQLException {
		//  couple_class_obj is like (idclass1=122 AND idobj1=12) OR (idclass1=122 AND idobj1=13) ......
		String query;
		if (couple_class_obj==null){
			couple_class_obj="";
		} else if (couple_class_obj.equals("")){
			
		} else {
			couple_class_obj=" WHERE "+couple_class_obj;
		}
		query = "SELECT idclass1,idobj1,domaindescription,classdescription,fieldcode,fielddescription FROM relationlist";
		query += couple_class_obj;
		if (!couple_class_obj.equals("")){
			query+=" AND status='A' "; 
		} else {
			query+=" WHERE status='A' ";
		}
		query += " ORDER BY \"idclass1\",\"idobj1\" ";
		Vector relationTipsVector = new Vector();
		System.out.println("Query SELECTRELATIONTIPS="+query);
		ResultSet rs = null;
		PreparedStatement stm = null;
		RelationBean rb = null;
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while ( rs.next() ) {
					rb = new RelationBean();
					rb.setClassId(rs.getInt(1));
					rb.setObjId(rs.getInt(2));
					rb.setDomainDescription(rs.getString(3));
					rb.setClassDescription(rs.getString(4));
					rb.setCode(rs.getString(5));
					rb.setDescription(rs.getString(6));
					relationTipsVector.add(rb);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return relationTipsVector;
	}
	public  Vector selectRelations(int IdClass, int rowId, int limit, int offset) throws SQLException {
		Vector relationVector = new Vector();
		RelationBean bean = null;
		ResultSet rs = null;
		PreparedStatement stm = null;
		// retrieve information of the other object
		String query = "select class1,class2,fieldcode,fielddescription,realname,iddomain,idclass1,idobj1,idclass2,idobj2,status,domaindescription,domaincardinality,classdescription,direct from relationlist where idclass1="+IdClass+" AND idobj1="+rowId+" AND status='A' ";
		if (limit!=0){
			query+= " LIMIT "+limit;
		}
		if (offset!=0){
			query+= " OFFSET "+offset;
		}
		System.out.println("Query SELECTRELATION="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while ( rs.next() ) { // while ( rs.next() ) {
					//bean.setClassName(rs.getString(1));
					bean = setRelationObject(rs);
					relationVector.add(bean);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return relationVector;
	}
	public Vector selectHistoryRelations(int IdClass, int rowId) throws SQLException {
		Vector relationVector = new Vector();
		HistoryBean his = null;
		ResultSet rs = null;
		PreparedStatement stm = null;
		// retrieve information of the other object
		//String query = "select class1,class2,fieldcode,fielddescription,realname,iddomain,idclass1,idobj1,idclass2,idobj2,status,domaindescription,domaincardinality,classdescription,direct from relationlist where idclass1="+IdClass+" AND idobj1="+rowId+" AND status<>'A' ";
		String query = "SELECT idclass2,idobj2,domaindescription,fieldcode,fielddescription,username,begindate,enddate FROM relationhistorylist WHERE idclass1="+IdClass+" AND idobj1="+rowId+" AND status<>'A' ";	
		System.out.println("Query SELECTHISTORYRELATION="+query);
		Connection con = db.getConnection();
		try {
			if ( con != null ) {
				stm = con.prepareStatement(query);
				rs = stm.executeQuery();
				while ( rs.next() ) { // while ( rs.next() ) {
					//bean.setClassName(rs.getString(1));
					his = new HistoryBean();
					his.setClassId(rs.getInt(1));
					his.setObjId(rs.getInt(2));
					his.setRelationName(rs.getString(3));
					his.setRelatedObjectCode(rs.getString(4));
					his.setRelatedObjectDescription(rs.getString(5));
					his.setUser(rs.getString(6));
					his.setBeginDate(rs.getString(7));
					his.setEndDate(rs.getString(8));
					his.setAttribute(false);
					relationVector.add(his);
				}
			}
		} catch( SQLException ex ) {
			throw ex;
		} finally {
			DBService.close(rs, stm, con);
		}
		return relationVector;
	}


	/**************************/
	/* USEFUL SETTING METHOTS */
	/**************************/
	
	private DomainBean setDomainObject(ResultSet rs) {
		DomainBean bean = new DomainBean();
		String nam ="";
		try {
			bean.setId(rs.getInt(1));
			nam = rs.getString(2);
			if (nam.indexOf("Map_")!=-1){
				nam = nam.substring(4);
			}
			bean.setName(nam);
			bean.setDirectDescription(rs.getString(3));
			bean.setInverseDescription(rs.getString(4));
			bean.setStartClass(rs.getString(5));
			bean.setEndClass(rs.getString(6));
			bean.setCardinality(rs.getString(7));
			bean.setStatus(rs.getString(8));
		} catch (SQLException e) {
			e.printStackTrace();
		}		
		return bean;
	}

	private RelationBean setRelationObject(ResultSet rs) {
		RelationBean bean = new RelationBean();		
		try {
			bean.setClassName(rs.getString(2));
			bean.setCode(rs.getString(3));
			bean.setDescription(rs.getString(4));
			bean.setDomainRealName(rs.getString(5));
			bean.setDomainId(rs.getInt(6));
			bean.setClassId(rs.getInt(9));
			bean.setObjId(rs.getInt(10));
			bean.setStatus(rs.getString(11));
			bean.setDomainDescription(rs.getString(12));
			// cardinality 13
			bean.setClassDescription(rs.getString(14));
			bean.setDirect(rs.getBoolean(15));
		} catch (SQLException e) {
			e.printStackTrace();
		}		
		return bean;
	}
	
} // end of class
